MODULE jyv_func
 
! From the book "Computation of Special Functions"
!      by Shanjie Zhang and Jianming Jin
!   Copyright 1996 by John Wiley & Sons, Inc.
! The authors state:
!   "However, we give permission to the reader who purchases this book
!    to incorporate any of these programs into his or her programs
!    provided that the copyright is acknowledged."
 
IMPLICIT NONE
INTEGER, PARAMETER  :: dp = SELECTED_REAL_KIND(12, 60)
 
CONTAINS
 

SUBROUTINE jyv(v, x, vm, bj, dj, by, dy)

!    =======================================================
!    Purpose: Compute Bessel functions Jv(x) and Yv(x)
!             and their derivatives
!    Input :  x --- Argument of Jv(x) and Yv(x)
!             v --- Order of Jv(x) and Yv(x)
!                   ( v = n+v0, 0  v0 < 1, n = 0,1,2,... )
!    Output:  BJ(n) --- Jn+v0(x)
!             DJ(n) --- Jn+v0'(x)
!             BY(n) --- Yn+v0(x)
!             DY(n) --- Yn+v0'(x)
!             VM --- Highest order computed
!    Routines called:
!         (1) GAMMA for computing gamma function
!         (2) MSTA1 and MSTA2 for computing the starting
!             point for backward recurrence
!    =======================================================

REAL (dp), INTENT(IN)   :: v
REAL (dp), INTENT(IN)   :: x
REAL (dp), INTENT(OUT)  :: vm
REAL (dp), INTENT(OUT)  :: bj(0:)
REAL (dp), INTENT(OUT)  :: dj(0:)
REAL (dp), INTENT(OUT)  :: by(0:)
REAL (dp), INTENT(OUT)  :: dy(0:)

REAL (dp), PARAMETER  :: el = .5772156649015329_dp, pi = 3.141592653589793_dp, &
                         rp2 = .63661977236758_dp
REAL (dp)  :: a, a0, b, bju0, bju1, bjv0, bjv1, bjvl, byv0, byv1, byvk,  &
              ck, cs, cs0, cs1, ec, f, f0, f1, f2, ga, gb, pv0, pv1, px,  &
              qx, r, r0, r1, rp, rq, sk, v0, vg, vl, vv, w0, w1, x2, xk
INTEGER    :: j, k, k0, l, m, n

x2 = x * x
n = v
v0 = v - n
IF (x < 1.0D-100) THEN
  DO  k = 0, n
    bj(k) = 0.0D0
    dj(k) = 0.0D0
    by(k) = -1.0D+300
    dy(k) = 1.0D+300
  END DO
  IF (v0 == 0.0) THEN
    bj(0) = 1.0D0
    dj(1) = 0.5D0
  ELSE
    dj(0) = 1.0D+300
  END IF
  vm = v
  RETURN
END IF
IF (x <= 12.0) THEN
  DO  l = 0, 1
    vl = v0 + l
    bjvl = 1.0D0
    r = 1.0D0
    DO  k = 1, 40
      r = -0.25D0 * r * x2 / (k*(k+vl))
      bjvl = bjvl + r
      IF (ABS(r) < ABS(bjvl)*1.0D-15) EXIT
    END DO
    vg = 1.0D0 + vl
    CALL gamma(vg, ga)
    a = (0.5D0*x) ** vl / ga
    IF (l == 0) bjv0 = bjvl * a
    IF (l == 1) bjv1 = bjvl * a
  END DO
ELSE
  k0 = 11
  IF (x >= 35.0) k0 = 10
  IF (x >= 50.0) k0 = 8
  DO  j = 0, 1
    vv = 4.0D0 * (j+v0) * (j+v0)
    px = 1.0D0
    rp = 1.0D0
    DO  k = 1, k0
      rp = -0.78125D-2 * rp * (vv - (4*k-3)**2) * (vv - (4*k-1)**2) /  &
           (k*(2*k-1)*x2)
      px = px + rp
    END DO
    qx = 1.0D0
    rq = 1.0D0
    DO  k = 1, k0
      rq = -0.78125D-2 * rq * (vv - (4*k-1)**2) * (vv - (4*k+1)**2) /   &
           (k*(2*k+1)*x2)
      qx = qx + rq
    END DO
    qx = 0.125D0 * (vv-1.0) * qx / x
    xk = x - (0.5D0*(j+v0) + 0.25D0) * pi
    a0 = SQRT(rp2/x)
    ck = COS(xk)
    sk = SIN(xk)
    IF (j == 0) THEN
      bjv0 = a0 * (px*ck - qx*sk)
      byv0 = a0 * (px*sk + qx*ck)
    ELSE IF (j == 1) THEN
      bjv1 = a0 * (px*ck - qx*sk)
      byv1 = a0 * (px*sk + qx*ck)
    END IF
  END DO
END IF
bj(0) = bjv0
bj(1) = bjv1
dj(0) = v0 / x * bj(0) - bj(1)
dj(1) = -(1.0D0+v0) / x * bj(1) + bj(0)
IF (n >= 2 .AND. n <= INT(0.9*x)) THEN
  f0 = bjv0
  f1 = bjv1
  DO  k = 2, n
    f = 2.0D0 * (k+v0-1.0D0) / x * f1 - f0
    bj(k) = f
    f0 = f1
    f1 = f
  END DO
ELSE IF (n >= 2) THEN
  m = msta1(x, 200)
  IF (m < n) THEN
    n = m
  ELSE
    m = msta2(x, n, 15)
  END IF
  f2 = 0.0D0
  f1 = 1.0D-100
  DO  k = m, 0, -1
    f = 2.0D0 * (v0+k+1) / x * f1 - f2
    IF (k <= n) bj(k) = f
    f2 = f1
    f1 = f
  END DO
  IF (ABS(bjv0) > ABS(bjv1)) THEN
    cs = bjv0 / f
  ELSE
    cs = bjv1 / f2
  END IF
  bj(0:n) = cs * bj(0:n)
END IF
DO  k = 2, n
  dj(k) = -(k+v0) / x * bj(k) + bj(k-1)
END DO
IF (x <= 12.0D0) THEN
  IF (v0 /= 0.0) THEN
    DO  l = 0, 1
      vl = v0 + l
      bjvl = 1.0D0
      r = 1.0D0
      DO  k = 1, 40
        r = -0.25D0 * r * x2 / (k*(k-vl))
        bjvl = bjvl + r
        IF (ABS(r) < ABS(bjvl)*1.0D-15) EXIT
      END DO
      vg = 1.0D0 - vl
      CALL gamma(vg,gb)
      b = (2.0D0/x) ** vl / gb
      IF (l == 0) bju0 = bjvl * b
      IF (l == 1) bju1 = bjvl * b
    END DO
    pv0 = pi * v0
    pv1 = pi * (1.0D0+v0)
    byv0 = (bjv0*COS(pv0) - bju0) / SIN(pv0)
    byv1 = (bjv1*COS(pv1) - bju1) / SIN(pv1)
  ELSE
    ec = LOG(x/2.0D0) + el
    cs0 = 0.0D0
    w0 = 0.0D0
    r0 = 1.0D0
    DO  k = 1, 30
      w0 = w0 + 1.0D0 / k
      r0 = -0.25D0 * r0 / (k*k) * x2
      cs0 = cs0 + r0 * w0
    END DO
    byv0 = rp2 * (ec*bjv0 - cs0)
    cs1 = 1.0D0
    w1 = 0.0D0
    r1 = 1.0D0
    DO  k = 1, 30
      w1 = w1 + 1.0D0 / k
      r1 = -0.25D0 * r1 / (k*(k+1)) * x2
      cs1 = cs1 + r1 * (2.0D0*w1+1.0D0/(k+1.0D0))
    END DO
    byv1 = rp2 * (ec*bjv1 - 1.0D0/x - 0.25D0*x*cs1)
  END IF
END IF
by(0) = byv0
by(1) = byv1
DO  k = 2, n
  byvk = 2.0D0 * (v0+k-1) / x * byv1 - byv0
  by(k) = byvk
  byv0 = byv1
  byv1 = byvk
END DO
dy(0) = v0 / x * by(0) - by(1)
DO  k = 1, n
  dy(k) = -(k+v0) / x * by(k) + by(k-1)
END DO
vm = n + v0
RETURN
END SUBROUTINE jyv



SUBROUTINE gamma(x, ga)

!    ==================================================
!    Purpose: Compute gamma function (x)
!    Input :  x  --- Argument of (x)
!                    ( x is not equal to 0,-1,-2,)
!    Output:  GA --- (x)
!    ==================================================

REAL (dp), INTENT(IN)   :: x
REAL (dp), INTENT(OUT)  :: ga

REAL (dp), PARAMETER  :: g(26) = (/ 1.0_dp, 0.5772156649015329_dp,  &
     -0.6558780715202538_dp, -0.420026350340952D-1, 0.1665386113822915_dp,   &
     -0.421977345555443D-1, -.96219715278770D-2, .72189432466630D-2,  &
     -0.11651675918591D-2, -.2152416741149D-3, .1280502823882D-3,  &
     -0.201348547807D-4, -.12504934821D-5, .11330272320D-5, -.2056338417D-6, &
      0.61160950D-8, .50020075D-8, -.11812746D-8, .1043427D-9, .77823D-11,  &
      -.36968D-11, .51D-12, -.206D-13, -.54D-14, .14D-14, .1D-15 /)
REAL (dp), PARAMETER  :: pi = 3.141592653589793_dp
REAL (dp)  :: gr, r, z
INTEGER    :: k, m, m1

IF (x == INT(x)) THEN
  IF (x > 0.0_dp) THEN
    ga = 1.0_dp
    m1 = x - 1
    DO  k = 2, m1
      ga = ga * k
    END DO
  ELSE
    ga = 1.0D+300
  END IF
ELSE
  IF (ABS(x) > 1.0_dp) THEN
    z = ABS(x)
    m = z
    r = 1.0_dp
    DO  k = 1, m
      r = r * (z-k)
    END DO
    z = z - m
  ELSE
    z = x
  END IF
  gr = g(26)
  DO  k = 25, 1, -1
    gr = gr * z + g(k)
  END DO
  ga = 1.0_dp / (gr*z)
  IF (ABS(x) > 1.0_dp) THEN
    ga = ga * r
    IF (x < 0.0_dp) ga = -pi / (x*ga*SIN(pi*x))
  END IF
END IF
RETURN
END SUBROUTINE gamma



FUNCTION msta1(x, mp) RESULT(fn_val)

!    ===================================================
!    Purpose: Determine the starting point for backward
!             recurrence such that the magnitude of
!             Jn(x) at that point is about 10^(-MP)
!    Input :  x     --- Argument of Jn(x)
!             MP    --- Value of magnitude
!    Output:  MSTA1 --- Starting point
!    ===================================================

REAL (dp), INTENT(IN)  :: x
INTEGER, INTENT(IN)    :: mp
INTEGER                :: fn_val

REAL (dp)  :: a0, f, f0, f1
INTEGER    :: it, n0, n1, nn

a0 = ABS(x)
n0 = INT(1.1*a0) + 1
f0 = envj(n0,a0) - mp
n1 = n0 + 5
f1 = envj(n1,a0) - mp
DO  it = 1, 20
  nn = n1 - (n1-n0) / (1.0_dp - f0/f1)
  f = envj(nn,a0) - mp
  IF (ABS(nn-n1) < 1) EXIT
  n0 = n1
  f0 = f1
  n1 = nn
  f1 = f
END DO

fn_val = nn
RETURN
END FUNCTION msta1



FUNCTION msta2(x, n, mp) RESULT(fn_val)

!    ===================================================
!    Purpose: Determine the starting point for backward
!             recurrence such that all Jn(x) has MP
!             significant digits
!    Input :  x  --- Argument of Jn(x)
!             n  --- Order of Jn(x)
!             MP --- Significant digit
!    Output:  MSTA2 --- Starting point
!    ===================================================

REAL (dp), INTENT(IN)  :: x
INTEGER, INTENT(IN)    :: n
INTEGER, INTENT(IN)    :: mp
INTEGER                :: fn_val

REAL (dp)  :: a0, ejn, f, f0, f1, hmp, obj
INTEGER    :: it, n0, n1, nn

a0 = ABS(x)
hmp = 0.5_dp * mp
ejn = envj(n, a0)
IF (ejn <= hmp) THEN
  obj = mp
  n0 = INT(1.1*a0)
ELSE
  obj = hmp + ejn
  n0 = n
END IF
f0 = envj(n0,a0) - obj
n1 = n0 + 5
f1 = envj(n1,a0) - obj
DO  it = 1, 20
  nn = n1 - (n1-n0) / (1.0_dp - f0/f1)
  f = envj(nn, a0) - obj
  IF (ABS(nn-n1) < 1) EXIT
  n0 = n1
  f0 = f1
  n1 = nn
  f1 = f
END DO

fn_val = nn + 10
RETURN
END FUNCTION msta2



FUNCTION envj(n, x) RESULT(fn_val)

INTEGER, INTENT(IN)    :: n
REAL (dp), INTENT(IN)  :: x
REAL (dp)              :: fn_val

fn_val = 0.5_dp * LOG10(6.28_dp*n) - n * LOG10(1.36_dp*x/n)
RETURN
END FUNCTION envj

END MODULE jyv_func
 
 
 
PROGRAM mjyv
USE jyv_func
IMPLICIT NONE

! Code converted using TO_F90 by Alan Miller
! Date: 2001-12-25  Time: 11:55:42

!    ============================================================
!    Purpose: This program computes Bessel functions Jv(x) and
!             Yv(x) and their derivatives using subroutine JYV
!    Input :  x --- Argument of Jv(x) and Yv(x)
!             v --- Order of Jv(x) and Yv(x)
!                   ( v = n+v0,  0  n  250, 0  v0 < 1 )
!    Output:  BJ(n) --- Jn+v0(x)
!             DJ(n) --- Jn+v0'(x)
!             BY(n) --- Yn+v0(x)
!             DY(n) --- Yn+v0'(x)
!    Example: Compute Jv(x) and Yv(x) and their derivatives
!             for v = 0.25(1.0)5.25 and x = 10.0
!             Computation results:

!             v =  5.25,      x = 10.00

!     v        Jv(x)         Jv'(x)        Yv(x)         Yv'(x)
!    ------------------------------------------------------------
!     .25   -.20639379    -.13476340     .14493044    -.21381777
!    1.25    .12960355    -.22259423     .21744103     .11775031
!    2.25    .23879467     .07587475    -.09057018     .23781932
!    3.25   -.02214595     .24599211    -.25819761    -.00665596
!    4.25   -.25318954     .08545961    -.07725827    -.22536285
!    5.25   -.19306516    -.15183033     .19252809    -.17833551
!    ============================================================

REAL (dp)  :: bj(0:250), dj(0:250), by(0:250), dy(0:250), v, v0, vk, vm, x
INTEGER    :: k, nm, ns

WRITE (*,*) '  Please enter v, x '
READ (*,*) v, x
WRITE (*,5100) v, x
IF (v <= 8) THEN
  ns = 1
ELSE
  WRITE (*,*) '  Please enter order step Ns'
  READ (*,*) ns
END IF
CALL jyv(v, x, vm, bj, dj, by, dy)
nm = vm
v0 = vm - nm
WRITE(*,*)
WRITE(*,*)'   v         Jv(x)           Jv''(X)          Yv(x)           Yv''(X)'
WRITE(*,*)' ---------------------------------------------------------------------'
DO  k = 0, nm, ns
  vk = k + v0
  WRITE (*,5000) vk, bj(k), dj(k), by(k), dy(k)
END DO
STOP

5000 FORMAT (' ', f6.2, 4g16.8)
5100 FORMAT (t9, 'v =', f6.2, ',    x =', f6.2)
END PROGRAM mjyv
