MODULE pbdv_func
 
! From the book "Computation of Special Functions"
!      by Shanjie Zhang and Jianming Jin
!   Copyright 1996 by John Wiley & Sons, Inc.
! The authors state:
!   "However, we give permission to the reader who purchases this book
!    to incorporate any of these programs into his or her programs
!    provided that the copyright is acknowledged."
 
IMPLICIT NONE
INTEGER, PARAMETER  :: dpr = SELECTED_REAL_KIND(12, 60)
 
CONTAINS
 

SUBROUTINE pbdv(v, x, dv, dp, pdf, pdd)

!    ====================================================
!    Purpose: Compute parabolic cylinder functions Dv(x)
!             and their derivatives
!    Input:   x --- Argument of Dv(x)
!             v --- Order of Dv(x)
!    Output:  DV(na) --- Dn+v0(x)
!             DP(na) --- Dn+v0'(x)
!             ( na = |n|, v0 = v-n, |v0| < 1,
!               n = 0,1,2, )
!             PDF --- Dv(x)
!             PDD --- Dv'(x)
!    Routines called:
!          (1) DVSA for computing Dv(x) for small |x|
!          (2) DVLA for computing Dv(x) for large |x|
!    ====================================================

REAL (dpr), INTENT(IN OUT)  :: v
REAL (dpr), INTENT(IN)      :: x
REAL (dpr), INTENT(OUT)     :: dv(0:)
REAL (dpr), INTENT(OUT)     :: dp(0:)
REAL (dpr), INTENT(OUT)     :: pdf
REAL (dpr), INTENT(OUT)     :: pdd

REAL (dpr)  :: ep, f, f0, f1, pd, pd0, pd1, s0, v0, v1, v2, vh, xa
INTEGER     :: ja, k, l, m, na, nk, nv

xa = ABS(x)
vh = v
v = v + SIGN(1.0D0,v)
nv = v
v0 = v - nv
na = ABS(nv)
ep = EXP(-.25D0*x*x)
IF (na >= 1) ja = 1
IF (v >= 0.0) THEN
  IF (v0 == 0.0) THEN
    pd0 = ep
    pd1 = x * ep
  ELSE
    DO  l = 0, ja
      v1 = v0 + l
      IF (xa <= 5.8) CALL dvsa(v1,x,pd1)
      IF (xa > 5.8) CALL dvla(v1,x,pd1)
      IF (l == 0) pd0 = pd1
    END DO
  END IF
  dv(0) = pd0
  dv(1) = pd1
  DO  k = 2, na
    pdf = x * pd1 - (k+v0-1.0D0) * pd0
    dv(k) = pdf
    pd0 = pd1
    pd1 = pdf
  END DO
ELSE
  IF (x <= 0.0) THEN
    IF (xa <= 5.8D0) THEN
      CALL dvsa(v0,x,pd0)
      v1 = v0 - 1.0D0
      CALL dvsa(v1, x, pd1)
    ELSE
      CALL dvla(v0, x, pd0)
      v1 = v0 - 1.0D0
      CALL dvla(v1,x,pd1)
    END IF
    dv(0) = pd0
    dv(1) = pd1
    DO  k = 2, na
      pd = (-x*pd1 + pd0) / (k-1.0D0-v0)
      dv(k) = pd
      pd0 = pd1
      pd1 = pd
    END DO
  ELSE IF (x <= 2.0) THEN
    v2 = nv + v0
    IF (nv == 0) v2 = v2 - 1.0D0
    nk = INT(-v2)
    CALL dvsa(v2,x,f1)
    v1 = v2 + 1.0D0
    CALL dvsa(v1,x,f0)
    dv(nk) = f1
    dv(nk-1) = f0
    DO  k = nk - 2, 0, -1
      f = x * f0 + (k-v0+1.0D0) * f1
      dv(k) = f
      f1 = f0
      f0 = f
    END DO
  ELSE
    IF (xa <= 5.8) CALL dvsa(v0,x,pd0)
    IF (xa > 5.8) CALL dvla(v0,x,pd0)
    dv(0) = pd0
    m = 100 + na
    f1 = 0.0D0
    f0 = 1.0D-30
    DO  k = m, 0, -1
      f = x * f0 + (k-v0+1.0D0) * f1
      IF (k <= na) dv(k) = f
      f1 = f0
      f0 = f
    END DO
    s0 = pd0 / f
    DO  k = 0, na
      dv(k) = s0 * dv(k)
    END DO
  END IF
END IF
DO  k = 0, na - 1
  v1 = ABS(v0) + k
  IF (v >= 0.0D0) THEN
    dp(k) = 0.5D0 * x * dv(k) - dv(k+1)
  ELSE
    dp(k) = -0.5D0 * x * dv(k) - v1 * dv(k+1)
  END IF
END DO
pdf = dv(na-1)
pdd = dp(na-1)
v = vh
RETURN
END SUBROUTINE pbdv


SUBROUTINE dvsa(va, x, pd)

!    ===================================================
!    Purpose: Compute parabolic cylinder function Dv(x)
!             for small argument
!    Input:   x  --- Argument
!             va --- Order
!    Output:  PD --- Dv(x)
!    Routine called: GAMMA for computing (x)
!    ===================================================

REAL (dpr), INTENT(IN)   :: va
REAL (dpr), INTENT(IN)   :: x
REAL (dpr), INTENT(OUT)  :: pd

REAL (dpr)  :: a0, ep, eps, g0, g1, ga0, gm, pi, r, r1,  sq2, va0, vm, vt
INTEGER     :: m

eps = 1.0D-15
pi = 3.141592653589793_dpr
sq2 = SQRT(2.0D0)
ep = EXP(-.25D0*x*x)
va0 = 0.5D0 * (1.0D0-va)
IF (va == 0.0) THEN
  pd = ep
ELSE
  IF (x == 0.0) THEN
    IF (va0 <= 0.0 .AND. va0 == INT(va0)) THEN
      pd = 0.0D0
    ELSE
      CALL gamma(va0,ga0)
      pd = SQRT(pi) / (2.0D0**(-.5D0*va)*ga0)
    END IF
  ELSE
    CALL gamma(-va,g1)
    a0 = 2.0D0 ** (-0.5D0*va-1.0D0) * ep / g1
    vt = -.5D0 * va
    CALL gamma(vt,g0)
    pd = g0
    r = 1.0D0
    DO  m = 1, 250
      vm = .5D0 * (m-va)
      CALL gamma(vm,gm)
      r = -r * sq2 * x / m
      r1 = gm * r
      pd = pd + r1
      IF (ABS(r1) < ABS(pd)*eps) EXIT
    END DO
    pd = a0 * pd
  END IF
END IF
RETURN
END SUBROUTINE dvsa


SUBROUTINE dvla(va, x, pd)

!    ====================================================
!    Purpose: Compute parabolic cylinder functions Dv(x)
!             for large argument
!    Input:   x  --- Argument
!             va --- Order
!    Output:  PD --- Dv(x)
!    Routines called:
!          (1) VVLA for computing Vv(x) for large |x|
!          (2) GAMMA for computing (x)
!    ====================================================

REAL (dpr), INTENT(IN)   :: va
REAL (dpr), INTENT(IN)   :: x
REAL (dpr), INTENT(OUT)  :: pd

REAL (dpr), PARAMETER  :: pi = 3.141592653589793_dpr
REAL (dpr)  :: a0, ep, eps, gl, r, vl, x1
INTEGER     :: k

eps = 1.0D-12
ep = EXP(-.25*x*x)
a0 = ABS(x) ** va * ep
r = 1.0D0
pd = 1.0D0
DO  k = 1, 16
  r = -0.5D0 * r * (2.0*k-va-1.0) * (2.0*k-va-2.0) / (k*x*x)
  pd = pd + r
  IF (ABS(r/pd) < eps) EXIT
END DO
pd = a0 * pd
IF (x < 0.0D0) THEN
  x1 = -x
  CALL vvla(va, x1, vl)
  CALL gamma(-va, gl)
  pd = pi * vl / gl + COS(pi*va) * pd
END IF
RETURN
END SUBROUTINE dvla


SUBROUTINE vvla(va, x, pv)

!    ===================================================
!    Purpose: Compute parabolic cylinder function Vv(x)
!             for large argument
!    Input:   x  --- Argument
!             va --- Order
!    Output:  PV --- Vv(x)
!    Routines called:
!          (1) DVLA for computing Dv(x) for large |x|
!          (2) GAMMA for computing (x)
!    ===================================================

REAL (dpr), INTENT(IN)   :: va
REAL (dpr), INTENT(IN)   :: x
REAL (dpr), INTENT(OUT)  :: pv

REAL (dpr), PARAMETER  :: pi = 3.141592653589793_dpr
REAL (dpr)  :: a0, dsl, eps, gl, pdl, qe, r, x1
INTEGER     :: k

eps = 1.0D-12
qe = EXP(0.25*x*x)
a0 = ABS(x) ** (-va-1.0D0) * SQRT(2.0D0/pi) * qe
r = 1.0D0
pv = 1.0D0
DO  k = 1, 18
  r = 0.5D0 * r * (2.0*k+va-1.0) * (2.0*k+va) / (k*x*x)
  pv = pv + r
  IF (ABS(r/pv) < eps) EXIT
END DO
pv = a0 * pv
IF (x < 0.0D0) THEN
  x1 = -x
  CALL dvla(va, x1, pdl)
  CALL gamma(-va, gl)
  dsl = SIN(pi*va) **2
  pv = dsl * gl / pi * pdl - COS(pi*va) * pv
END IF
RETURN
END SUBROUTINE vvla


SUBROUTINE gamma(x, ga)

!    ==================================================
!    Purpose: Compute the gamma function (x)
!    Input :  x  --- Argument of (x)
!                    ( x is not equal to 0,-1,-2, )
!    Output:  GA --- (x)
!    ==================================================

REAL (dpr), INTENT(IN)   :: x
REAL (dpr), INTENT(OUT)  :: ga

REAL (dpr), PARAMETER  :: g(26) = (/  &
       1.0_dpr, 0.5772156649015329_dpr, -0.6558780715202538_dpr,  &
      -0.420026350340952D-1, 0.1665386113822915_dpr,   &
      -0.421977345555443D-1, -.96219715278770D-2,  &
      .72189432466630D-2, -.11651675918591D-2, -.2152416741149D-3,  &
      .1280502823882D-3, -.201348547807D-4, -.12504934821D-5,  &
      .11330272320D-5, -.2056338417D-6, .61160950D-8,  &
      .50020075D-8, -.11812746D-8, .1043427D-9, .77823D-11,  &
      -.36968D-11, .51D-12, -.206D-13, -.54D-14, .14D-14, .1D-15 /)
REAL (dpr), PARAMETER  :: pi = 3.141592653589793_dpr
REAL (dpr)  :: gr, r, z
INTEGER     :: k, m, m1

IF (x == INT(x)) THEN
  IF (x > 0.0_dpr) THEN
    ga = 1.0_dpr
    m1 = x - 1
    DO  k = 2, m1
      ga = ga * k
    END DO
  ELSE
    ga = 1.0D+300
  END IF
ELSE
  IF (ABS(x) > 1.0_dpr) THEN
    z = ABS(x)
    m = INT(z)
    r = 1.0_dpr
    DO  k = 1, m
      r = r * (z-k)
    END DO
    z = z - m
  ELSE
    z = x
  END IF
  gr = g(26)
  DO  k = 25, 1, -1
    gr = gr * z + g(k)
  END DO
  ga = 1.0_dpr / (gr*z)
  IF (ABS(x) > 1.0_dpr) THEN
    ga = ga * r
    IF (x < 0.0_dpr) ga = -pi / (x*ga*SIN(pi*x))
  END IF
END IF
RETURN
END SUBROUTINE gamma

END MODULE pbdv_func
 
 
 
PROGRAM mpbdv
USE pbdv_func
IMPLICIT NONE

! Code converted using TO_F90 by Alan Miller
! Date: 2001-12-25  Time: 11:55:44

!    ===============================================================
!    Purpose: This program computes the parabolic cylinder functions
!             Dv(x) and their derivatives using subroutine PBDV
!    Input:   x --- Argument of Dv(x)
!             v --- Order of Dv(x)
!    Output:  DV(na) --- Dn+v0(x)
!             DP(na) --- Dn+v0'(x)
!             ( na = |n|, n = int(v), v0 = v-n, |v0| < 1
!               n = 0,1,2,, |n|  100 )
!             PDF --- Dv(x)
!             PDD --- Dv'(x)
!    Example: v = 5.5,  x =10.0,  v0 = 0.5,  n = 0,1,...,5

!               n+v0      Dv(x)           Dv'(x)
!             ---------------------------------------
!               0.5   .43971930D-10  -.21767183D-09
!               1.5   .43753148D-09  -.21216995D-08
!               2.5   .43093569D-08  -.20452956D-07
!               3.5   .41999741D-07  -.19491595D-06
!               4.5   .40491466D-06  -.18355745D-05
!               5.5   .38601477D-05  -.17073708D-04

!             Dv(x)= .38601477D-05,  Dv'(x)=-.17073708D-04
!    ===============================================================

REAL (dpr)  :: dv(0:100), dp(0:100), pdd, pdf, v, v0, vk, x
INTEGER     :: k, na, nv

WRITE (*,*) 'Please enter v and  x '
READ (*,*) v, x
WRITE (*,5000) v, x
nv = v
v0 = v - nv
na = ABS(nv)
CALL pbdv(v, x, dv, dp, pdf, pdd)
WRITE (*,*)
WRITE (*,*) '   v       Dv(x)           Dv''(X)'
WRITE (*,*) '---------------------------------------'
DO  k = 0, na
  vk = k * SIGN(1,nv) + v0
  WRITE (*,5100) vk, dv(k), dp(k)
END DO
WRITE (*,*)
WRITE (*,5200) v, pdf, pdd
STOP

5000 FORMAT (' v =', f6.2,',    x =', f6.2)
5100 FORMAT (' ', f5.1, 2g16.8)
5200 FORMAT (' v =', f5.1, ',  Dv(x)=', g14.8,',   Dv''(X)=', g14.8)
END PROGRAM mpbdv
