MODULE stvhv_func
 
! From the book "Computation of Special Functions"
!      by Shanjie Zhang and Jianming Jin
!   Copyright 1996 by John Wiley & Sons, Inc.
! The authors state:
!   "However, we give permission to the reader who purchases this book
!    to incorporate any of these programs into his or her programs
!    provided that the copyright is acknowledged."
 
IMPLICIT NONE
INTEGER, PARAMETER  :: dp = SELECTED_REAL_KIND(12, 60)
 
CONTAINS
 

SUBROUTINE stvhv(v, x, hv)

!    =====================================================
!    Purpose: Compute Struve function Hv(x) with an
!             arbitrary order v
!    Input :  v  --- Order of Hv(x)  ( -8.0  v  12.5 )
!             x  --- Argument of Hv(x) ( x  0 )
!    Output:  HV --- Hv(x)
!    Routine called: GAMMA to compute the gamma function
!    =====================================================

REAL (dp), INTENT(IN)   :: v
REAL (dp), INTENT(IN)   :: x
REAL (dp), INTENT(OUT)  :: hv

REAL (dp), PARAMETER  :: pi = 3.141592653589793_dp
REAL (dp)  :: bf, bf0, bf1, by0, by1, byv, ga, gb, pu0, pu1, qu0, qu1,  &
              r1, r2, s, s0, sa, sr, t0, t1, u, u0, v0, v2, va, vb, vt
INTEGER    :: k, l, n

IF (x == 0.0_dp) THEN
  IF (v > -1.0 .OR. INT(v)-v == 0.5_dp) THEN
    hv = 0.0_dp
  ELSE IF (v < -1.0_dp) THEN
    hv = (-1) ** (INT(0.5_dp-v)-1) * 1.0D+300
  ELSE IF (v == -1.0_dp) THEN
    hv = 2.0_dp / pi
  END IF
  RETURN
END IF
IF (x <= 20.0_dp) THEN
  v0 = v + 1.5_dp
  CALL gamma(v0, ga)
  s = 2.0_dp / (SQRT(pi)*ga)
  r1 = 1.0_dp
  DO  k = 1, 100
    va = k + 1.5_dp
    CALL gamma(va, ga)
    vb = v + k + 1.5_dp
    CALL gamma(vb, gb)
    r1 = -r1 * (0.5_dp*x) ** 2
    r2 = r1 / (ga*gb)
    s = s + r2
    IF (ABS(r2) < ABS(s)*1.0D-12) EXIT
  END DO
  hv = (0.5_dp*x) ** (v+1.0_dp) * s
ELSE
  sa = (0.5_dp*x) ** (v-1.0) / pi
  v0 = v + 0.5_dp
  CALL gamma(v0, ga)
  s = SQRT(pi) / ga
  r1 = 1.0_dp
  DO  k = 1, 12
    va = k + 0.5_dp
    CALL gamma(va, ga)
    vb = -k + v + 0.5_dp
    CALL gamma(vb, gb)
    r1 = r1 / (0.5_dp*x) ** 2
    s = s + r1 * ga / gb
  END DO
  s0 = sa * s
  u = ABS(v)
  n = u
  u0 = u - n
  DO  l = 0, 1
    vt = 4.0_dp * (u0+l) ** 2
    r1 = 1.0_dp
    pu1 = 1.0_dp
    DO  k = 1, 12
      r1 = -0.0078125_dp * r1 * (vt-(4*k-3)**2) * (vt-(4*k-1)**2) / ((2*k-1)*k*x*x)
      pu1 = pu1 + r1
    END DO
    qu1 = 1.0_dp
    r2 = 1.0_dp
    DO  k = 1, 12
      r2 = -0.0078125_dp * r2 * (vt-(4*k-1)**2) * (vt-(4*k+1)**2) / ((2*k+1)*k*x*x)
      qu1 = qu1 + r2
    END DO
    qu1 = 0.125_dp * (vt-1.0_dp) / x * qu1
    IF (l == 0) THEN
      pu0 = pu1
      qu0 = qu1
    END IF
  END DO
  t0 = x - (0.5*u0 + 0.25_dp) * pi
  t1 = x - (0.5*u0 + 0.75_dp) * pi
  sr = SQRT(2.0_dp/(pi*x))
  by0 = sr * (pu0*SIN(t0) + qu0*COS(t0))
  by1 = sr * (pu1*SIN(t1) + qu1*COS(t1))
  bf0 = by0
  bf1 = by1
  DO  k = 2, n
    bf = 2.0_dp * (k-1.0+u0) / x * bf1 - bf0
    bf0 = bf1
    bf1 = bf
  END DO
  IF (n == 0) byv = by0
  IF (n == 1) byv = by1
  IF (n > 1) byv = bf
  hv = byv + s0
END IF
RETURN
END SUBROUTINE stvhv


SUBROUTINE gamma(x, ga)

!    ==================================================
!    Purpose: Compute the gamma function (x)
!    Input :  x  --- Argument of (x)
!                    ( x is not equal to 0,-1,-2, )
!    Output:  GA --- (x)
!    ==================================================

REAL (dp), INTENT(IN)   :: x
REAL (dp), INTENT(OUT)  :: ga

REAL (dp), PARAMETER  :: g(26) = (/  &
       1.0_dp, 0.5772156649015329_dp, -0.6558780715202538_dp,  &
      -0.420026350340952D-1, 0.1665386113822915_dp,   &
      -0.421977345555443D-1, -.96219715278770D-2,  &
      .72189432466630D-2, -.11651675918591D-2, -.2152416741149D-3,  &
      .1280502823882D-3, -.201348547807D-4, -.12504934821D-5,  &
      .11330272320D-5, -.2056338417D-6, .61160950D-8,  &
      .50020075D-8, -.11812746D-8, .1043427D-9, .77823D-11,  &
      -.36968D-11, .51D-12, -.206D-13, -.54D-14, .14D-14, .1D-15 /)
REAL (dp), PARAMETER  :: pi = 3.141592653589793_dp
REAL (dp)  :: gr, r, z
INTEGER    :: k, m, m1

IF (x == INT(x)) THEN
  IF (x > 0.0_dp) THEN
    ga = 1.0_dp
    m1 = x - 1
    DO  k = 2, m1
      ga = ga * k
    END DO
  ELSE
    ga = 1.0D+300
  END IF
ELSE
  IF (ABS(x) > 1.0_dp) THEN
    z = ABS(x)
    m = INT(z)
    r = 1.0_dp
    DO  k = 1, m
      r = r * (z-k)
    END DO
    z = z - m
  ELSE
    z = x
  END IF
  gr = g(26)
  DO  k = 25, 1, -1
    gr = gr * z + g(k)
  END DO
  ga = 1.0_dp / (gr*z)
  IF (ABS(x) > 1.0_dp) THEN
    ga = ga * r
    IF (x < 0.0_dp) ga = -pi / (x*ga*SIN(pi*x))
  END IF
END IF
RETURN
END SUBROUTINE gamma

END MODULE stvhv_func
 
 
 
PROGRAM mstvhv
USE stvhv_func
IMPLICIT NONE

! Code converted using TO_F90 by Alan Miller
! Date: 2001-12-25  Time: 11:55:47

!    =======================================================
!    Purpose:  This program computes Struve function Hv(x)
!              for an arbitrary order using subroutine STVHV
!    Input :   v  --- Order of Hv(x)  ( -8.0  v  12.5 )
!              x  --- Argument of Hv(x) ( x  0 )
!    Output:   HV --- Hv(x)
!    Example:  x = 10.0
!                v           Hv(x)
!              -----------------------
!                .5     .46402212D+00
!               1.5     .14452322D+01
!               2.5     .31234632D+01
!               3.5     .53730255D+01
!               4.5     .72083122D+01
!               5.5     .76851132D+01
!    =======================================================

REAL (dp)  :: hv, v, x

WRITE (*,*) 'Please v and x '
READ (*,*) v, x
WRITE (*,5100) v, x
WRITE (*,*)
WRITE (*,*) '   v           Hv(x)'
WRITE (*,*) ' -----------------------'
CALL stvhv(v, x, hv)
WRITE (*,5000) v, hv
STOP

5000 FORMAT (' ', f5.1, g18.8)
5100 FORMAT (' v =', f5.1, '      x =', f5.1)
END PROGRAM mstvhv
